@@ -1,22 +1,21 @@
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+
+#     http://www.apache.org/licenses/LICENSE-2.0
+
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+
+# Created by: Thomas Oliver
+# Created date: 18th March 2021
+# Copyright 2021 InferStat Ltd
+
 """
 API facade that allows interaction with the library with strings and vanilla Python objects.
-
-Copyright 2021 InferStat Ltd
-
-Licensed under the Apache License, Version 2.0 (the "License");
-you may not use this file except in compliance with the License.
-You may obtain a copy of the License at
-
-    http://www.apache.org/licenses/LICENSE-2.0
-
-Unless required by applicable law or agreed to in writing, software
-distributed under the License is distributed on an "AS IS" BASIS,
-WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-See the License for the specific language governing permissions and
-limitations under the License.
-
-Created by: Thomas Oliver
-Created date: 18th March 2021
 """
 
 # Python standard library
@@ -204,13 +203,20 @@ class Api:
     def return_representations(
         name_of_algorithm: str, representation_or_list_of_representations: Union[str, List[str]] = None
     ) -> dict:
-        """Returns the representations (e.g. URLs of relevant documentation."""
+        """Returns the representations (e.g. URLs of relevant documentation)."""
+
+        dict_of_properties = Api.get_algorithm_information()
+        try:
+            representations = dict_of_properties[name_of_algorithm]["available_representation_types"]
+        except KeyError:
+            raise NotImplementedError("The requested algorithm does not have any representations: ", name_of_algorithm)
 
         if not representation_or_list_of_representations:
-            representation_or_list_of_representations = Api.get_available_representations()
+            representation_or_list_of_representations = Api.get_available_representations(name_of_algorithm)
+
+        if isinstance(representation_or_list_of_representations, str):
+            representation_or_list_of_representations = [representation_or_list_of_representations]
 
-        dict_of_properties = Api.get_algorithm_information(name_of_algorithm)
-        representations = dict_of_properties[name_of_algorithm]["available_representation_types"]
         if isinstance(representation_or_list_of_representations, list):
             representation_dict = {}
             for ii_entry in representation_or_list_of_representations:
@@ -218,13 +224,6 @@ class Api:
                     representation_dict.update({ii_entry: representations[ii_entry]})
                 else:
                     raise NameError("Could not find this representation: ", ii_entry)
-        elif isinstance(representation_or_list_of_representations, str):
-            try:
-                representation_dict = dict_of_properties[name_of_algorithm]["available_representation_types"][
-                    representation_or_list_of_representations
-                ]
-            except KeyError:
-                raise NameError("Could not find this representation: ", representation_or_list_of_representations)
         else:
             raise TypeError("Input type not supported: ", representation_or_list_of_representations)
 
